#define USERMENU_H

/****************************************************************

    Copyright (c) 1996-1997 Fred Pesch

    This program is free software; you can redistribute it 
    and/or modify it under the terms of the GNU General 
    Public License as published by the Free Software 
    Foundation (version 2 of the License).

    This program is distributed in the hope that it will 
    be useful, but WITHOUT ANY WARRANTY; without even the 
    implied warranty of MERCHANTABILITY or FITNESS FOR A 
    PARTICULAR PURPOSE.  See the GNU General Public License 
    for more details.

    For a copy of the GNU General Public License, write to the 
    Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
    MA 02139, USA.

****************************************************************/

#include <Xm/Xm.h>
#include <Xm/XmAll.h>
#include <stdio.h>

#include "interface.h"
#include "config.h"
#include "command.h"

struct MenuNode {
  TMenuType type;
  Widget w_node;
  Widget w_cascade;
  char *command;
  int nb_submenus;
  struct MenuNode **submenu;
};

typedef struct MenuNode TMenuNode;

TMenuNode *G_UserMenu=NULL;

TMenuNode *SearchMenuNode(Widget w, TMenuNode *node)
{
  int i;
  TMenuNode *result;

  switch(node->type) {
  case POPUP:
    if(node->nb_submenus==0) return(NULL);
    result=NULL;
    i=0;
    while(i<node->nb_submenus && result==NULL) {
      result=SearchMenuNode(w, node->submenu[i]);
      i++;
    }
    return(result);
  break;
  case MENUITEM:
    if(node->w_node==w) return(node);
    else return NULL;
    break;
  default:
    return NULL;
  }
}
      
void UserMenuCB(Widget w, XtPointer app_data, XtPointer widget_data)
{
  TMenuNode *activated;

  activated=SearchMenuNode(w, G_UserMenu);
  if(activated!=NULL && activated->command!=NULL)
    UserMenuCommandRoutine(activated->command);
}

TMenuNode *CreateSubMenu(TMenuNode *parent, TMenuEntry entry)
{
  TMenuNode *result;
  XmString strtemp;
  int i,j,found;

  result = (TMenuNode *) XtMalloc(sizeof(TMenuNode));
  switch(entry.type) {
  case SEPARATOR :
    result->type=SEPARATOR;
    result->w_node=XmCreateSeparator(parent->w_node, "separator", NULL, 0);
    XtManageChild(result->w_node);
    result->w_cascade=NULL;
    result->command=NULL;
    result->nb_submenus=0;
    result->submenu=NULL;
    break;
  case MENUITEM :
    result->type=MENUITEM;
    strtemp=XmStringCreateSimple(entry.name);
    result->w_node=XtVaCreateManagedWidget("button", xmPushButtonWidgetClass, parent->w_node, XmNlabelString, strtemp, NULL);
    XmStringFree(strtemp);
    XtAddCallback(result->w_node, XmNactivateCallback, UserMenuCB, NULL);
    result->w_cascade=NULL;
    result->command=XtMalloc(strlen(entry.command)+1);
    strcpy(result->command, entry.command);
    result->nb_submenus=0;
    result->submenu=NULL;
    break;
  case POPUP :
    result->type=POPUP;
    result->w_node=XmCreatePulldownMenu(parent->w_node, "popup", NULL, 0);
    strtemp=XmStringCreateSimple(entry.name);
    result->w_cascade=XtVaCreateManagedWidget("cascade", xmCascadeButtonWidgetClass, parent->w_node, XmNlabelString, strtemp, XmNsubMenuId, result->w_node, NULL);
    XmStringFree(strtemp);    
    result->command=NULL;
    i=0;
    found=0;
    while(i<G_NbMenus && !found) {
      if(strcmp(G_MenuList[i].ident,entry.name)==0) 
	found=1;
      else i++;
    }
    if(!found) { result->nb_submenus=0; result->submenu=NULL; return result; }
    result->nb_submenus=G_MenuList[i].nb_entries;
    result->submenu=(TMenuNode **) XtMalloc(sizeof(TMenuNode *)*result->nb_submenus);
    for(j=0;j<G_MenuList[i].nb_entries;j++)
      result->submenu[j]=CreateSubMenu(result, G_MenuList[i].entry[j]);
    break;
  default:
  }
  return(result); 
}

void CreateUserMenu(void)
{
  int i,j;
  int found;

  if(G_NbMenus==0) return;
  G_UserMenu=(TMenuNode *) XtMalloc(sizeof(TMenuNode));
  G_UserMenu->type=POPUP;
  G_UserMenu->w_node=XtNameToWidget(W_MainWindow, "*UserMenu");
  i=0;
  found=0;
  while(i<G_NbMenus && !found) {
    if(strcmp(G_MenuList[i].ident,"UserMenu")==0) 
      found=1;
    else i++;
  }
  if(!found) {
    XtFree((char *) G_UserMenu);
    G_UserMenu=NULL;
    return;
  }
  G_UserMenu->nb_submenus=G_MenuList[i].nb_entries;
  if(G_UserMenu->nb_submenus>0) {
    G_UserMenu->submenu=(TMenuNode **) XtMalloc(sizeof(TMenuNode *)*G_UserMenu->nb_submenus);
    for(j=0;j<G_MenuList[i].nb_entries;j++)
      G_UserMenu->submenu[j]=CreateSubMenu(G_UserMenu, G_MenuList[i].entry[j]);
  } else G_UserMenu->submenu=NULL;
}

void DestroySubMenu(TMenuNode *node)
{
  int i;

  if(node->w_node!=NULL) {
    XtUnmanageChild(node->w_node);
    XtDestroyWidget(node->w_node);
  }
  if(node->w_cascade!=NULL) {
    XtUnmanageChild(node->w_node);
    XtDestroyWidget(node->w_node);
  }
  if(node->command!=NULL) XtFree(node->command);
  if(node->nb_submenus!=0) {
    for(i=0;i<node->nb_submenus;i++) DestroySubMenu(node->submenu[i]);
    XtFree((char *) node->submenu);
  }
  XtFree((char *)node);
}

void DestroyUserMenu(void)
{
  int i;

  if(G_UserMenu==NULL) return;
  if(G_UserMenu->nb_submenus!=0) {
    for(i=0;i<G_UserMenu->nb_submenus;i++)
      DestroySubMenu(G_UserMenu->submenu[i]);
    XtFree((char *) G_UserMenu->submenu);
  }
  XtFree((char *) G_UserMenu);
  G_UserMenu=NULL;
}


